@@ -10,11 +10,11 @@
 # SPDX-License-Identifier: GPL-2.0-or-later
 # ------------------------------------------------------------------
 
-import os
-from pathlib import Path
+import pathlib
 
 import pytest
 
+from PyInstaller import isolated
 from PyInstaller.compat import is_darwin, is_linux, is_py39, is_win
 from PyInstaller.utils.hooks import is_module_satisfies, can_import_module, get_module_attribute
 from PyInstaller.utils.tests import importorskip, requires, xfail
@@ -462,9 +462,9 @@ def test_statsmodels(pyi_builder):
 @pytest.mark.skipif(not is_win, reason='pywin32-ctypes is supported only on Windows')
 @pytest.mark.parametrize('submodule', ['win32api', 'win32cred', 'pywintypes'])
 def test_pywin32ctypes(pyi_builder, submodule):
-    pyi_builder.test_source("""
-        from win32ctypes.pywin32 import {0}
-        """.format(submodule))
+    pyi_builder.test_source(f"""
+        from win32ctypes.pywin32 import {submodule}
+        """)
 
 
 @importorskip('pyproj')
@@ -1113,6 +1113,48 @@ def test_pygraphviz_bundled_programs(pyi_builder):
         """)
 
 
+@importorskip("pygraphviz")
+def test_pygraphviz_functional(pyi_builder, tmp_path):
+    # Functional test for pygraphviz that tries to use different programs and output formats to ensure that graphviz
+    # programs and plugins are properly collected.
+    pyi_builder.test_source("""
+        import sys
+        import os
+        import pygraphviz as pgv
+
+        output_dir = sys.argv[1] if len(sys.argv) >= 2 else '.'
+
+        print("Setting up graph...")
+        G = pgv.AGraph(strict=False, directed=True)
+
+        # Set default node attributes
+        G.graph_attr["label"] = "Name of graph"
+        G.node_attr["shape"] = "circle"
+        G.edge_attr["color"] = "red"
+
+        G.add_node("a")
+        G.add_edge("b", "c")  # add edge (and the nodes)
+
+        print("Dumping graph to string...")
+        s = G.string()
+        print(s)
+
+        print("Test layout with default program (= neato)")
+        G.layout()
+
+        print("Test layout with 'dot' program")
+        G.layout(prog="dot")
+
+        print("Writing previously positioned graph to PNG file...")
+        G.draw(os.path.join(output_dir, "file.png"))
+
+        print("Using 'circo' to position, writing PS file...")
+        G.draw(os.path.join(output_dir, "file.ps"), prog="circo")
+
+        print("Done!")
+        """, app_args=[str(tmp_path)])
+
+
 @importorskip("pypsexec")
 def test_pypsexec(pyi_builder):
     pyi_builder.test_source("""
@@ -1334,8 +1376,8 @@ def test_pyqtgraph_remote_graphics_view(pyi_builder):
 
 
 @importorskip('hydra')
-def test_hydra(pyi_builder, tmpdir):
-    config_file = str((Path(__file__) / '../data/test_hydra/config.yaml').resolve(strict=True).as_posix())
+def test_hydra(pyi_builder):
+    config_file = pathlib.Path(__file__).parent / 'data' / 'test_hydra' / 'config.yaml'
 
     pyi_builder.test_source(
         """
@@ -1354,7 +1396,7 @@ def test_hydra(pyi_builder, tmpdir):
         if __name__ == "__main__":
             my_app()
         """,
-        pyi_args=['--add-data', os.pathsep.join((config_file, 'conf'))]
+        pyi_args=['--add-data', f"{config_file}:conf"]
     )
 
 
@@ -1440,7 +1482,7 @@ def test_compliance_checker(pyi_builder):
     # The test file - taken from the package's own test data/examples. Use an .nc file instead of .cdl one, because
     # loading the latter requires ncgen utility to be available on the system.
     pkg_path = get_module_attribute('compliance_checker', '__path__')[0]
-    input_file = Path(pkg_path) / 'tests/data/bad-trajectory.nc'
+    input_file = pathlib.Path(pkg_path) / 'tests' / 'data' / 'bad-trajectory.nc'
     assert input_file.is_file(), f"Selected test file, {input_file!s} does not exist! Fix the test!"
 
     pyi_builder.test_source("""
@@ -2054,6 +2096,18 @@ def test_xarray_chunk(pyi_builder):
     """)
 
 
+# Test that we can import `dask.array` when `jinja2` is also available. In this case, `dask.array.core` ends up loading
+# template files from `dask.widgets.templates`. Requires `numpy` (or `dask[array]`) for `dask.array` to be importable,
+# and `jinja2` (`dask[diagnostics]`) for template files to become mandatory.
+@importorskip('dask')
+@importorskip('jinja2')
+@importorskip('numpy')
+def test_dask_array(pyi_builder):
+    pyi_builder.test_source("""
+        import dask.array
+    """)
+
+
 @importorskip('tables')
 def test_pytables(pyi_builder):
     # NOTE: run_from_path=True prevents `pyi_builder` from completely clearing the `PATH` environment variable. At the
@@ -2315,8 +2369,8 @@ def test_toga(pyi_builder):
 
 
 @importorskip('numbers_parser')
-def test_numbers_parser(pyi_builder, tmpdir):
-    output_filename = tmpdir / "output.numbers"
+def test_numbers_parser(pyi_builder, tmp_path):
+    output_filename = tmp_path / "output.numbers"
     pyi_builder.test_source("""
         import sys
         import numbers_parser
@@ -2334,6 +2388,110 @@ def test_numbers_parser(pyi_builder, tmpdir):
     """, app_args=[str(output_filename)])
 
 
+@importorskip('fsspec')
+def test_fsspec_protocols(pyi_builder, tmp_path):
+    # Get the list of working protocols in unfrozen python
+    @isolated.decorate
+    def _get_working_fsspec_protocols():
+        import fsspec
+
+        working_protocols = []
+        for protocol in fsspec.available_protocols():
+            try:
+                fsspec.get_filesystem_class(protocol)
+                working_protocols.append(protocol)
+            except ImportError:
+                pass
+
+        return sorted(working_protocols)
+
+    protocols_unfrozen = _get_working_fsspec_protocols()
+    print(f"Unfrozen protocols: {protocols_unfrozen}")
+
+    # Obtain list of working protocols in frozen application.
+    output_file = tmp_path / "output.txt"
+
+    pyi_builder.test_source("""
+        import sys
+        import fsspec
+
+        working_protocols = []
+        for protocol in fsspec.available_protocols():
+            try:
+                obj = fsspec.get_filesystem_class(protocol)
+                working_protocols.append(protocol)
+            except ImportError:
+                pass
+
+        with open(sys.argv[1], 'w') as fp:
+            for protocol in working_protocols:
+                print(f"{protocol}", file=fp)
+    """, app_args=[str(output_file)])
+
+    with open(output_file, "r") as fp:
+        protocols_frozen = sorted(line.strip() for line in fp)
+    print(f"Frozen protocols: {protocols_frozen}")
+
+    assert protocols_frozen == protocols_unfrozen
+
+
+@importorskip('zarr')
+@importorskip('xarray')
+def test_xarray_to_zarr(pyi_builder):
+    pyi_builder.test_source("""
+        import xarray as xr
+        import numpy as np
+
+        data = xr.DataArray(
+            np.random.randn(2, 3),
+            dims=("x", "y"),
+            coords={"x": [10, 20]},
+        )
+        data_zarr = data.to_zarr()
+        print(data)
+    """)
+
+
+@importorskip('intake')
+def test_intake(pyi_builder):
+    pyi_builder.test_source("""
+        import intake
+
+        print(f"intake version: {intake.__version__}")
+        catalog = intake.Catalog()
+    """)
+
+
+@importorskip('intake')
+def test_intake_plugins(pyi_builder, tmp_path):
+    # Get the list of plugins in unfrozen python
+    @isolated.decorate
+    def _get_intake_plugins():
+        import intake
+        return sorted(list(intake.registry))
+
+    plugins_unfrozen = _get_intake_plugins()
+    print(f"Unfrozen plugins: {plugins_unfrozen}")
+
+    # Obtain list of plugins in frozen application.
+    output_file = tmp_path / "output.txt"
+
+    pyi_builder.test_source("""
+        import sys
+        import intake
+
+        with open(sys.argv[1], 'w') as fp:
+            for plugin in intake.registry:
+                print(f"{plugin}", file=fp)
+    """, app_args=[str(output_file)])
+
+    with open(output_file, "r") as fp:
+        plugins_frozen = sorted(line.strip() for line in fp)
+    print(f"Frozen plugins: {plugins_frozen}")
+
+    assert plugins_frozen == plugins_unfrozen
+
+
 @importorskip('h3')
 def test_h3(pyi_builder):
     pyi_builder.test_source("""
@@ -2346,3 +2504,18 @@ def test_selectolax(pyi_builder):
     pyi_builder.test_source("""
         import selectolax
     """)
+
+
+@importorskip('ruamel.yaml')
+@pytest.mark.skipif(
+    not is_module_satisfies('ruamel.yaml.string'),
+    reason='ruamel.yaml.string plugin is not installed',
+)
+def test_ruamel_yaml_string_plugin(pyi_builder):
+    pyi_builder.test_source("""
+        import ruamel.yaml
+
+        yaml = ruamel.yaml.YAML(typ=['rt', 'string'])
+        data  = dict(abc=42, help=['on', 'its', 'way'])
+        print(yaml.dump_to_string(data))
+    """)
